@@ -17,6 +17,25 @@ log=logging.getLogger('model')
 if ctypes.c_char_p.__name__ == 'c_char_p':
   ctypes.original_c_char_p = ctypes.c_char_p
 
+''' keep orig class '''
+if ctypes.Structure.__name__ == 'Structure':
+  ctypes.original_Structure = ctypes.Structure
+
+__refs = list()
+__register = dict()
+
+def keepRef(obj):
+  ''' Sometypes, your have to cast a c_void_p, You can keep ref in Ctypes object, 
+    they might be transient (if obj == somepointer.contents).'''
+  __refs.append(obj)
+  return
+
+def register(klass):
+  klass.classRef = __register
+  __register[ctypes.POINTER(klass)] = klass
+  return klass
+
+
 ''' returns if the address of the struct is in the mapping area
 '''
 def is_valid_address(obj,mappings, structType=None):
@@ -188,6 +207,7 @@ class NotNullComparable:
 NotNull=NotNullComparable()
 
 class CString(ctypes.Union):
+  ''' ctypes.c_char_p can not be used for memory parsing. it tries to load the string itself '''
   _fields_=[
   ("string", ctypes.original_c_char_p),
   ("ptr", ctypes.POINTER(ctypes.c_ubyte) )
@@ -198,17 +218,6 @@ class CString(ctypes.Union):
     return self.string
   pass
 
-class EVP_CIPHER_CTX_APP_DATA(ctypes.c_ubyte):
-#ByteArray=ctypes.c_ubyte*1
-#class EVP_CIPHER_CTX_APP_DATA(ByteArray):
-  #_field_=[('_length_=1
-#  _length_=1024
-#  _type_=ctypes.c_ubyte
-  pass
-
-EVP_CIPHER_CTX_APP_DATA_PTR=ctypes.POINTER(EVP_CIPHER_CTX_APP_DATA)
-#MYEVP_CTX_PTR=ctypes.POINTER(ctypes.c_ubyte)
-#class EVP_CIPHER_CTX_APP_DATA_PTR(ctypes.POINTER(ctypes.c_ubyte)):
 
 
 #debug
@@ -327,7 +336,7 @@ class LoadableMembers(ctypes.Structure):
             return False
           log.debug('%s %s %s isNULL and that is OK'%(attrname,attrtype,repr(attr) ))
           return True
-      if ( not is_valid_address_value( myaddress, mappings) ) :
+      if (myaddress != 0) and ( not is_valid_address_value( myaddress, mappings) )   :
         log.debug('%s %s %s 0x%lx INVALID'%(attrname,attrtype, repr(attr) ,myaddress))
         return False
       log.debug('%s %s %s is at 0x%lx OK'%(attrname,attrtype,repr(attr),myaddress ))
@@ -379,10 +388,14 @@ class LoadableMembers(ctypes.Structure):
 
   def loadMembers(self, mappings, maxDepth):
     ''' 
-    isValid() should have been tested before, otherwise.. it's gonna fail...
-    we copy memory from process for each pointer
-    and assign it to a python object _here, then we assign 
-    the member to be a pointer to _here'''
+    The validity of the memebrs will be assessed.
+    Each members that can be ( structures, pointers), will be evaluated for validity and loaded recursively.
+    
+    @param mappings: list of memoryMappings for the process.
+    @param maxDepth: limitation of depth after which the loading/validation will stop and return results.
+
+    @returns True if everything has been loaded, False if something went wrong. 
+    '''
     if maxDepth == 0:
       log.warning('Maximum depth reach. Not loading any deeper members.')
       log.warning('Struct partially LOADED. %s not loaded'%(self.__class__.__name__))
@@ -450,9 +463,12 @@ class LoadableMembers(ctypes.Structure):
       return True
     # we have PointerType here . Basic or complex
     # exception cases
-    if isCStringPointer(attr):
+    if isCStringPointer(attr) : 
       # can't use basic c_char_p because we can't load in foreign memory
       attr_obj_address = getaddress(attr.ptr)
+      if not bool(attr_obj_address):
+        log.debug('%s %s is a CString, the pointer is null (validation must have occurred earlier) '%(attrname, attr))
+        return True
       memoryMap = is_valid_address_value(attr_obj_address, mappings)
       if not memoryMap :
         log.warning('Error on addr while fetching a CString. should not happen')
@@ -499,6 +515,7 @@ class LoadableMembers(ctypes.Structure):
     return True
   
   def toString(self,prefix=''):
+    ''' return a string formatted description of this Structure. '''
     s="%s # %s\n"%(prefix,repr(self) )
     _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
     for field,typ in _fieldsTuple:
@@ -580,6 +597,8 @@ class LoadableMembers(ctypes.Structure):
     return s
     
   def toPyObject(self):
+    ''' returns a Plain Old python object as a perfect copy of this ctypes object.
+    '''
     # get self class.
     #log.info("%s %s %s_py"%(self.__class__.__module__, sys.modules[self.__class__.__module__], self.__class__.__name__) )
     my_class=getattr(sys.modules[self.__class__.__module__],"%s_py"%(self.__class__.__name__) )
@@ -625,6 +644,15 @@ class LoadableMembers(ctypes.Structure):
 
 
 class pyObj(object):
+  ''' Base class for a plain old python object.
+  all haystack/ctypes classes will be translated in this format before pickling.
+  
+  Operations :
+    - toString(self, prefix):  print a nicely formatted data structure
+        @param prefix: str to insert before each line (\t after that)
+    - findCtypes(self) : checks if a ctypes is to be found somewhere is the object.
+                      Useful to check if the object can be pickled.
+  '''
   def toString(self, prefix=''):
     s='{\n'
     for attrname,typ in self.__dict__.items():
@@ -652,7 +680,7 @@ class pyObj(object):
     ret = False
     for attrname,typ in self.__dict__.items():
       attr = getattr(self, attrname)
-      if self._attrFindCtypes(self, attr, attrname,typ ):
+      if self._attrFindCtypes(attr, attrname,typ ):
         log.warning('Found a ctypes in %s'%(attrname))
         ret = True
 
@@ -674,39 +702,95 @@ class pyObj(object):
     return ret
 
 def findCtypesInPyObj(obj):
+  ''' check function to help in unpickling errors correction '''
   ret = False
-  if type(obj) is tuple or type(obj) is list:
+  if isinstance(obj, pyObj):
+    if obj.findCtypes():
+      log.warning('Found a ctypes in array/tuple')
+      return True
+  elif type(obj) is tuple or type(obj) is list:
     for el in obj:
-      if el.findCtypes():
+      if findCtypesInPyObj(el):
         log.warning('Found a ctypes in array/tuple')
         return True
-  obj.findCtypes()
+  return False
       
 import inspect,sys
 
-def pasteLoadableMemberMethodsOn(klass):
-  for n,func in inspect.getmembers(LoadableMembers, inspect.ismethod):
-    #setattr(klass, n, types.MethodType(func, klass) )  # bad self type. need LoadableMembers not Struct
-    # Py2.x
-    setattr(klass, n, func.im_func)
-  if not hasattr(klass, 'expectedValues' ):
-    setattr( klass, 'expectedValues', dict() )
-  if not hasattr(klass, 'classRef' ):
-    setattr( klass, 'classRef', dict() )
-  return klass
-
-''' Load all model classes and create a similar non-ctypes Python class  
-  thoses will be used to translate non pickable ctypes into POPOs.
-'''
-def createPOPO(modulename):
-  for klass,typ in inspect.getmembers(sys.modules[modulename], inspect.isclass):
-    if typ.__module__ == modulename:
-      setattr(sys.modules[modulename], '%s_py'%(klass), type('%s_py'%(klass),(pyObj,),{}) )
-      ##print sys.modules[modulename], '%s_py'%(klass)
-
-createPOPO(__name__)
+def copyGeneratedClasses(src, dst):
+  ''' 
+    Copies the members of a generated module into a classic module.
+    Name convention : 
+    generated: ctypes_libraryname_generated.py
+    classic  : ctypes_libraryname.py
+    
+  @param me : dst module
+  @param src : src module, generated
+  '''
+  __root_module_name,__dot,__module_name = dst.__name__.rpartition('.')
+  _loaded=0
+  _registered=0
+  for (name, klass) in inspect.getmembers(src, inspect.isclass):
+    if type(klass) == type(ctypes.Structure):
+      if klass.__module__.endswith('%s_generated'%(__module_name) ) :
+        setattr(dst, name, klass)
+        _loaded+=1
+    else:
+      #log.debug("%s - %s"%(name, klass))
+      pass
+  log.debug('loaded %d C structs from %s structs'%( _loaded, src.__name__))
+  log.debug('registered %d Pointers types'%( _registered))
+  log.debug('There is %d members in %s'%(len(src.__dict__), src.__name__))
+  return 
 
 
-''' replace c_char_p '''
+def createPOPOClasses( targetmodule ):
+  ''' Load all model classes and create a similar non-ctypes Python class  
+    thoses will be used to translate non pickable ctypes into POPOs.
+  '''
+  _created=0
+  for klass,typ in inspect.getmembers(targetmodule, inspect.isclass):
+    if typ.__module__.startswith(targetmodule.__name__):
+      kpy = type('%s_py'%(klass),( pyObj ,),{})
+      # we have to keep a local (model) ref because the class is being created here.
+      # and we have a targetmodule ref. because it's asked.
+      # and another ref on the real module for the basic type, because, that is probably were it's gonna be used.
+      setattr(sys.modules[__name__], '%s_py'%(klass), kpy )
+      setattr(targetmodule, '%s_py'%(klass), kpy )
+      _created+=1
+      if typ.__module__ != targetmodule.__name__: # copy also to generated
+        setattr(sys.modules[typ.__module__], '%s_py'%(klass), kpy )
+        #log.debug("Created %s_py"%klass)
+  log.debug('created %d POPO types'%( _created))
+  return
+
+def registerModule( targetmodule ):
+  ''' register a ctypes module. To be run by target module.
+      all members is module will be registered, against their pointer types,
+      in a lookup table
+      Creates POPO's to be able to unpickle ctypes.
+  '''
+  _registered = 0
+  for klass,typ in inspect.getmembers(targetmodule, inspect.isclass):
+    if typ.__module__.startswith(targetmodule.__name__):
+      register( typ )
+      _registered += 1
+  # create POPO's
+  createPOPOClasses( targetmodule )
+  log.debug('registered %d types'%( _registered))
+  return
+
+# create local POPO ( lodableMembers )
+createPOPOClasses(sys.modules[__name__] )
+# register LoadableMembers 
+register(LoadableMembers)
+
+
+''' replace c_char_p - it can handle memory parsing without reading it '''
 if ctypes.c_char_p.__name__ == 'c_char_p':
   ctypes.c_char_p = CString
+
+''' switch class - we need our methods on ctypes.Structures for generated classes to work  '''
+if ctypes.Structure.__name__ == 'Structure':
+  ctypes.Structure = LoadableMembers
+
